@@ -23,10 +23,6 @@
 
 #include <libgda/gda-quark-list.h>
 #include <libgda/gda-util.h>
-#include <glib/ghash.h>
-#include <glib/gmem.h>
-#include <glib/gmessages.h>
-#include <glib/gstrfuncs.h>
 
 struct _GdaQuarkList {
 	GHashTable *hash_table;
@@ -47,13 +43,6 @@ GType gda_quark_list_get_type (void)
  * Private functions
  */
 
-static void
-free_hash_pair (gpointer key, gpointer value, gpointer user_data)
-{
-	g_free (key);
-	g_free (value);
-}
-
 static void
 copy_hash_pair (gpointer key, gpointer value, gpointer user_data)
 {
@@ -78,7 +67,7 @@ gda_quark_list_new (void)
 	GdaQuarkList *qlist;
 
 	qlist = g_new0 (GdaQuarkList, 1);
-	qlist->hash_table = g_hash_table_new (g_str_hash, g_str_equal);
+	qlist->hash_table = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
 	return qlist;
 }
@@ -121,7 +110,7 @@ gda_quark_list_clear (GdaQuarkList *qlist)
 {
 	g_return_if_fail (qlist != NULL);
 	
-	g_hash_table_foreach_remove (qlist->hash_table, (GHRFunc) free_hash_pair, NULL);
+	g_hash_table_remove_all (qlist->hash_table);
 }
 
 /**
@@ -135,7 +124,6 @@ gda_quark_list_free (GdaQuarkList *qlist)
 {
 	g_return_if_fail (qlist != NULL);
 
-	gda_quark_list_clear (qlist);
 	g_hash_table_destroy (qlist->hash_table);
 
 	g_free (qlist);
@@ -188,10 +176,11 @@ gda_quark_list_add_from_string (GdaQuarkList *qlist, const gchar *string, gboole
 	gchar **arr;
 
 	g_return_if_fail (qlist != NULL);
-	g_return_if_fail (string != NULL);
+	if (!string || !*string)
+		return;
 
-	if (cleanup != FALSE)
-		gda_quark_list_clear(qlist);
+	if (cleanup)
+		gda_quark_list_clear (qlist);
 
 	arr = (gchar **) g_strsplit (string, ";", 0);
 	if (arr) {
@@ -204,11 +193,16 @@ gda_quark_list_add_from_string (GdaQuarkList *qlist, const gchar *string, gboole
 			if (pair && pair[0]) {
 				gchar *name = pair[0];
 				gchar *value = pair[1];
+				g_strstrip (name);
 				gda_rfc1738_decode (name);
+				g_strstrip (value);
 				gda_rfc1738_decode (value);
-				g_hash_table_insert (qlist->hash_table, (gpointer) name, (gpointer) value);
+				g_hash_table_insert (qlist->hash_table, 
+						     (gpointer) name, (gpointer) value);
 				g_free (pair);
 			}
+			else
+				g_strfreev (pair);
 			n++;
 		}
 		g_strfreev (arr);
@@ -247,18 +241,10 @@ gda_quark_list_find (GdaQuarkList *qlist, const gchar *name)
 void
 gda_quark_list_remove (GdaQuarkList *qlist, const gchar *name)
 {
-	gpointer orig_key;
-	gpointer orig_value;
-
 	g_return_if_fail (qlist != NULL);
 	g_return_if_fail (name != NULL);
 
-	if (g_hash_table_lookup_extended (qlist->hash_table, name,
-					  &orig_key, &orig_value)) {
-		g_hash_table_remove (qlist->hash_table, name);
-		g_free (orig_key);
-		g_free (orig_value);
-	}
+	g_hash_table_remove (qlist->hash_table, name);
 }
 
 /**
